﻿using ProjectMotorboat.Drownings;
using System.Text;

namespace ProjectMotorboat.CollectionGenericObjects;

public class StorageCollection<T>
    where T : DrawningBoat
{

    private Dictionary<string, ICollectionGenericObjects<T>> _storages;

    public List<string> Keys => _storages.Keys.ToList();

    public StorageCollection()
    {
        _storages = new Dictionary<string, ICollectionGenericObjects<T>>();
    }


    public void AddCollection(string name, CollectionType collectionType)
    {
       
        if (name == null || _storages.ContainsKey(name)) { return; }
        switch (collectionType)

        {
            case CollectionType.None:
                return;
            case CollectionType.Massive:
                _storages[name] = new MassiveGenericObjects<T>();
                return;
            case CollectionType.List:
                _storages[name] = new ListGenericObjects<T>();
                return;
        }
    }


    public void DelCollection(string name)
    {
       
        if (_storages.ContainsKey(name))
            _storages.Remove(name);
    }


    public ICollectionGenericObjects<T>? this[string name]
    {
        get
        {
           
            if (name == null || !_storages.ContainsKey(name)) { return null; }
            return _storages[name];
        }
    }

    private readonly string _collectionKey = "CollectionsStorage";

    private readonly string _separatorForKeyValue = "|";

    private readonly string _separatorItems = ";";

    
    public bool SaveData(string filename)
    {
        if (_storages.Count == 0)
        {
            return false;
        }

        if (File.Exists(filename))
        {
            File.Delete(filename);
        }

        using (StreamWriter writer = new StreamWriter(filename))
        {
            writer.Write(_collectionKey);
            foreach (KeyValuePair<string, ICollectionGenericObjects<T>> value in _storages)
            {
                StringBuilder sb = new();
                sb.Append(Environment.NewLine);
                
                if (value.Value.Count == 0)
                {
                    continue;
                }
                sb.Append(value.Key);
                sb.Append(_separatorForKeyValue);
                sb.Append(value.Value.GetCollectionType);
                sb.Append(_separatorForKeyValue);
                sb.Append(value.Value.MaxCount);
                sb.Append(_separatorForKeyValue);
                foreach (T? item in value.Value.GetItems())
                {
                    string data = item?.GetDataForSave() ?? string.Empty;
                    if (string.IsNullOrEmpty(data))
                    {
                        continue;
                    }
                    sb.Append(data);
                    sb.Append(_separatorItems);
                }
                writer.Write(sb);
            }

        }

        return true;
    }

   
    public bool LoadData(string filename)
    {
        if (!File.Exists(filename))
        {
            return false;
        }
        using (StreamReader fs = File.OpenText(filename))
        {
            string str = fs.ReadLine();
            if (str == null || str.Length == 0)
            {
                return false;
            }
            if (!str.StartsWith(_collectionKey))
            {
                return false;
            }
            _storages.Clear();
            string strs = "";
            while ((strs = fs.ReadLine()) != null)
            {
                string[] record = strs.Split(_separatorForKeyValue, StringSplitOptions.RemoveEmptyEntries);
                if (record.Length != 4)
                {
                    continue;
                }
                CollectionType collectionType = (CollectionType)Enum.Parse(typeof(CollectionType), record[1]);
                ICollectionGenericObjects<T>? collection = StorageCollection<T>.CreateCollection(collectionType);
                if (collection == null)
                {
                    return false;
                }
                collection.MaxCount = Convert.ToInt32(record[2]);
                string[] set = record[3].Split(_separatorItems, StringSplitOptions.RemoveEmptyEntries);
                foreach (string elem in set)
                {
                    if (elem?.CreateDrawningBoat() is T militaryAircraft)
                    {
                        if (collection.Insert(militaryAircraft) == -1)
                        {
                            return false;
                        }
                    }
                }
                _storages.Add(record[0], collection);
            }
            return true;
        }
    }
    private static ICollectionGenericObjects<T>? CreateCollection(CollectionType collectionType)
    {
        return collectionType switch
        {
            CollectionType.Massive => new MassiveGenericObjects<T>(),
            CollectionType.List => new ListGenericObjects<T>(),
            _ => null,
        };
    }
}
